#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <io.h>
#include <dos.h>
#include <conio.h>

#define FALSE         0
#define TRUE  (! FALSE)
#define maxlayers   131

/* maxlayers-1 = maximum number of layers in laminate */

struct matrix { double M[3][3]; }
	      A,B,D,
	      Q[maxlayers],
	      QB[maxlayers],
	      S[maxlayers],
	      SB[maxlayers];

double pi=3.141592655,
       total_thickness=0.0,
       midplane=0.0,
       KT=0.0,
       E11[maxlayers],
       E22[maxlayers],
       NU12[maxlayers],
       NU21[maxlayers],
       G12[maxlayers],
       layer_angle[maxlayers],
       thickness[maxlayers],
       top[maxlayers],
       bottom[maxlayers],
       midplane_to_bottom[maxlayers],
       midplane_to_top[maxlayers];

int    total_layers=0,
       current_layer=1;

char   laminate_def[255];

/* ----------------------------------------------------------------- */
double sqr(double value)
  {
    return(value*value);
  };

/* ----------------------------------------------------------------- */
double cube(double value)
  {
    return(value*value*value);
  };

/* ----------------------------------------------------------------- */
void compute_Q_matrices()
{
  int i;
  for (i=1;i<=total_layers;i++)
  {
    /* compute Q matrix for these properties */
    Q[i].M[0][0] = E11[i]/(1.0-(NU12[i]*NU21[i]));
    Q[i].M[1][1] = E22[i]/(1.0-(NU12[i]*NU21[i]));
    Q[i].M[0][1] = NU12[i]*E22[i]/(1.0-(NU12[i]*NU21[i]));
    Q[i].M[1][0] = Q[i].M[0][1];

    Q[i].M[2][2] = G12[i];

    Q[i].M[0][2] = 0.0;
    Q[i].M[2][0] = Q[i].M[0][2];

    Q[i].M[1][2] = 0.0;
    Q[i].M[2][1] = Q[i].M[1][2];
  };
}

/* ----------------------------------------------------------------- */
void compute_S_matrices()
{
  int i;
  for (i=1;i<=total_layers;i++)
  {
    /* compute S matrix for these properties */
    S[i].M[0][0] = 1.0/E11[i];
    S[i].M[1][1] = 1.0/E22[i];
    S[i].M[0][1] = -NU12[i]/E11[i];
    S[i].M[1][0] = S[i].M[0][1];

    S[i].M[2][2] = 1.0/G12[i];

    S[i].M[0][2] = 0.0;
    S[i].M[2][0] = S[i].M[0][2];

    S[i].M[1][2] = 0.0;
    S[i].M[2][1] = S[i].M[1][2];
  };
}

/* ----------------------------------------------------------------- */
void compute_QB_matrices()
{
  int i;
  double angle;

  /* compute QB matrix for these angles */
  for (i=1;i<=total_layers;i++)
  {
    angle=layer_angle[i];
    angle=angle*pi/180.0;  /* convert to radians */

    QB[i].M[0][0] = Q[i].M[0][0]*sqr(sqr(cos(angle))) +
	 2.0*(Q[i].M[0][1]+2.0*Q[i].M[2][2])*sqr(sin(angle))*sqr(cos(angle)) +
	 Q[i].M[1][1] * sqr(sqr(sin(angle)));

    QB[i].M[0][1] = Q[i].M[0][1]*(sqr(sqr(sin(angle))) + sqr(sqr(cos(angle)))) +
	 (Q[i].M[0][0]+Q[i].M[1][1]-4.0*Q[i].M[2][2])*sqr(sin(angle))*sqr(cos(angle));

    QB[i].M[1][0] = QB[i].M[0][1];

    QB[i].M[1][1] = Q[i].M[0][0]*sqr(sqr(sin(angle))) +
	 2.0*(Q[i].M[0][1]+2.0*Q[i].M[2][2])* sqr(sin(angle)) * sqr(cos(angle)) +
	 Q[i].M[1][1]*sqr(sqr(cos(angle)));

    QB[i].M[0][2] = (Q[i].M[0][0] - Q[i].M[0][1] - 2.0*Q[i].M[2][2])*sin(angle)*cos(angle)*sqr(cos(angle)) +
	 (Q[i].M[0][1] - Q[i].M[1][1] + 2.0*Q[i].M[2][2])*sin(angle)*sqr(sin(angle))*cos(angle);

    QB[i].M[2][0] = QB[i].M[0][2];

    QB[i].M[1][2] = (Q[i].M[0][0] - Q[i].M[0][1] - 2.0*Q[i].M[2][2])*sin(angle)*sqr(sin(angle))*cos(angle) +
	  (Q[i].M[0][1] - Q[i].M[1][1] + 2.0*Q[i].M[2][2])*sin(angle)*sqr(cos(angle))*cos(angle);

    QB[i].M[2][1] = QB[i].M[1][2];

    QB[i].M[2][2] = (Q[i].M[0][0]+Q[i].M[1][1]-2.0*Q[i].M[0][1]-2.0*Q[i].M[2][2])*sqr(sin(angle))*sqr(cos(angle)) +
	 Q[i].M[2][2]*(sqr(sqr(sin(angle))) + sqr(sqr(cos(angle))));
  };
}

/* ----------------------------------------------------------------- */
void compute_SB_matrices()
{
  int i;
  double angle;

  /* compute QB matrix for these angles */
  for (i=1;i<=total_layers;i++)
  {
    angle=layer_angle[i];
    angle=angle*pi/180.0;  /* convert to radians */

    SB[i].M[0][0] = S[i].M[0][0]*sqr(sqr(cos(angle))) +
	 2.0*(S[i].M[0][1]+S[i].M[2][2])*sqr(sin(angle))*sqr(cos(angle)) +
	 S[i].M[1][1] * sqr(sqr(sin(angle)));

    SB[i].M[0][1] = S[i].M[0][1]*(sqr(sqr(sin(angle))) + sqr(sqr(cos(angle)))) +
	 (S[i].M[0][0]+S[i].M[1][1]-S[i].M[2][2])*sqr(sin(angle))*sqr(cos(angle));

    SB[i].M[1][0] = SB[i].M[0][1];

    SB[i].M[1][1] = S[i].M[0][0]*sqr(sqr(sin(angle))) +
	 (2.0*S[i].M[0][1]+S[i].M[2][2])* sqr(sin(angle)) * sqr(cos(angle)) +
	 S[i].M[1][1]*sqr(sqr(cos(angle)));

    SB[i].M[0][2] = (2.0*S[i].M[0][0] - 2.0*S[i].M[0][1] - S[i].M[2][2])*sin(angle)*cos(angle)*sqr(cos(angle)) -
	 (2.0*S[i].M[0][1] - 2.0*S[i].M[1][1] - S[i].M[2][2])*sin(angle)*sqr(sin(angle))*cos(angle);

    SB[i].M[2][0] = SB[i].M[0][2];

    SB[i].M[1][2] =  (2.0*S[i].M[0][0] - 2.0*S[i].M[0][1] - S[i].M[2][2])*sin(angle)*sqr(sin(angle))*cos(angle) -
	  (2.0*S[i].M[0][1] - 2.0*S[i].M[1][1] + S[i].M[2][2])*sin(angle)*sqr(cos(angle))*cos(angle);

    SB[i].M[2][1] = SB[i].M[1][2];

    SB[i].M[2][2] = 2.0*(2.0*S[i].M[0][0]+2.0*S[i].M[1][1]-4.0*S[i].M[0][1]-S[i].M[2][2])*sqr(sin(angle))*sqr(cos(angle)) +
	 S[i].M[2][2]*(sqr(sqr(sin(angle))) + sqr(sqr(cos(angle))));
  };
}

/* ----------------------------------------------------------------- */
void compute_Hj_values()
{
  int i;

  /* define each Hj and Hj-1*/
  total_thickness = 0.0;
  if ((total_layers<1) || (total_layers>maxlayers)) exit(0);

  for (i=1;i<=total_layers;i++)
    total_thickness=total_thickness+thickness[i];
  midplane = total_thickness / 2.0;
  top[1] = 0.0;
  bottom[total_layers] = total_thickness;
  for (i=1;i<total_layers;i++)
  {
     bottom[i]             = top[i] + thickness[i];
     top[i+1]              = bottom[i];
     midplane_to_top[i]    = top[i] - midplane;
     midplane_to_bottom[i] = bottom[i] - midplane;
  };
  midplane_to_top[total_layers]    = top[total_layers] - midplane;
  midplane_to_bottom[total_layers] = bottom[total_layers] - midplane;
}

/* ----------------------------------------------------------------- */
void create_laminate_def()
{
  int i,j,pos;
  char tempstring[7];

  /* create laminate definition string [x/x/x/x] */
  laminate_def[0]='['; pos=1;
  for (i=1;i<=total_layers;i++)
  {
    sprintf(tempstring,"%4.1f",layer_angle[i]);
    for (j=0;j<strlen(tempstring);j++)
    {
      laminate_def[pos] = tempstring[j]; pos++;
    };
    laminate_def[pos] = '/'; pos++;
  };
  laminate_def[pos-1]=']';
  laminate_def[pos]='\0';
}

/* ----------------------------------------------------------------- */
void compute_ABD_matrices()
{
  int i;

  /* compute A matrix */
  A.M[0][0] = 0.0;
  A.M[0][1] = 0.0;
  A.M[1][1] = 0.0;
  A.M[0][2] = 0.0;
  A.M[1][2] = 0.0;
  A.M[2][2] = 0.0;
  for (i=1;i<=total_layers;i++)
  {
     A.M[0][0] = A.M[0][0] + QB[i].M[0][0] *
		(midplane_to_bottom[i] - midplane_to_top[i]);
     A.M[0][1] = A.M[0][1] + QB[i].M[0][1] *
		(midplane_to_bottom[i] - midplane_to_top[i]);
     A.M[1][1] = A.M[1][1] + QB[i].M[1][1] *
		(midplane_to_bottom[i] - midplane_to_top[i]);
     A.M[0][2] = A.M[0][2] + QB[i].M[0][2] *
		(midplane_to_bottom[i] - midplane_to_top[i]);
     A.M[1][2] = A.M[1][2] + QB[i].M[1][2] *
		(midplane_to_bottom[i] - midplane_to_top[i]);
     A.M[2][2] = A.M[2][2] + QB[i].M[2][2] *
		(midplane_to_bottom[i] - midplane_to_top[i]);
     A.M[1][0] = A.M[0][1];
     A.M[2][0] = A.M[0][2];
     A.M[2][1] = A.M[1][2];
  };

  /* compute B matrix for this laminate */
  B.M[0][0] = 0.0;
  B.M[0][1] = 0.0;
  B.M[1][1] = 0.0;
  B.M[0][2] = 0.0;
  B.M[1][2] = 0.0;
  B.M[2][2] = 0.0;
  for (i=1;i<=total_layers;i++)
  {
     B.M[0][0] = B.M[0][0] + QB[i].M[0][0] *
		(sqr(midplane_to_bottom[i]) - sqr(midplane_to_top[i]));
     B.M[0][1] = B.M[0][1] + QB[i].M[0][1] *
		(sqr(midplane_to_bottom[i]) - sqr(midplane_to_top[i]));
     B.M[1][1] = B.M[1][1] + QB[i].M[1][1] *
		(sqr(midplane_to_bottom[i]) - sqr(midplane_to_top[i]));
     B.M[0][2] = B.M[0][2] + QB[i].M[0][2] *
		(sqr(midplane_to_bottom[i]) - sqr(midplane_to_top[i]));
     B.M[1][2] = B.M[1][2] + QB[i].M[1][2] *
		(sqr(midplane_to_bottom[i]) - sqr(midplane_to_top[i]));
     B.M[2][2] = B.M[2][2] + QB[i].M[2][2] *
		(sqr(midplane_to_bottom[i]) - sqr(midplane_to_top[i]));
  };
  B.M[0][0] = B.M[0][0] * 0.5;
  B.M[0][1] = B.M[0][1] * 0.5;
  B.M[1][1] = B.M[1][1] * 0.5;
  B.M[0][2] = B.M[0][2] * 0.5;
  B.M[1][2] = B.M[1][2] * 0.5;
  B.M[2][2] = B.M[2][2] * 0.5;

  B.M[1][0] = B.M[0][1];
  B.M[2][0] = B.M[0][2];
  B.M[2][1] = B.M[1][2];

  /* compute D matrix for this laminate */
  D.M[0][0] = 0.0;
  D.M[0][1] = 0.0;
  D.M[1][1] = 0.0;
  D.M[0][2] = 0.0;
  D.M[1][2] = 0.0;
  D.M[2][2] = 0.0;
  for (i=1;i<=total_layers;i++)
  {
     D.M[0][0] = D.M[0][0] + QB[i].M[0][0] *
		(cube(midplane_to_bottom[i]) - cube(midplane_to_top[i]));
     D.M[0][1] = D.M[0][1] + QB[i].M[0][1] *
		(cube(midplane_to_bottom[i]) - cube(midplane_to_top[i]));
     D.M[1][1] = D.M[1][1] + QB[i].M[1][1] *
		(cube(midplane_to_bottom[i]) - cube(midplane_to_top[i]));
     D.M[0][2] = D.M[0][2] + QB[i].M[0][2] *
		(cube(midplane_to_bottom[i]) - cube(midplane_to_top[i]));
     D.M[1][2] = D.M[1][2] + QB[i].M[1][2] *
		(cube(midplane_to_bottom[i]) - cube(midplane_to_top[i]));
     D.M[2][2] = D.M[2][2] + QB[i].M[2][2] *
		(cube(midplane_to_bottom[i]) - cube(midplane_to_top[i]));
  };
  D.M[0][0] = D.M[0][0] * (1.0/3.0);
  D.M[0][1] = D.M[0][1] * (1.0/3.0);
  D.M[1][1] = D.M[1][1] * (1.0/3.0);
  D.M[0][2] = D.M[0][2] * (1.0/3.0);
  D.M[1][2] = D.M[1][2] * (1.0/3.0);
  D.M[2][2] = D.M[2][2] * (1.0/3.0);

  D.M[1][0] = D.M[0][1];
  D.M[2][0] = D.M[0][2];
  D.M[2][1] = D.M[1][2];
}

/* ----------------------------------------------------------------- */
void Get_ENTER()
{
  char dum;

  printf("\nPress ENTER to continue...");
  dum=' ';
  while (dum!='\n') dum=getc(stdin);
}

/* ----------------------------------------------------------------- */
void print_matrix(FILE *output,struct matrix my_matrix,char *exp)
{
  fprintf(output," +--                                  --+\n");
  fprintf(output," |                                      |\n");
  fprintf(output," |  %10.3f  %10.3f  %10.3f  |\n",
		  my_matrix.M[0][0],my_matrix.M[0][1],my_matrix.M[0][2]);
  fprintf(output," |                                      |\n");
  fprintf(output," |  %10.3f  %10.3f  %10.3f  |    %s\n",
		  my_matrix.M[1][0],my_matrix.M[1][1],my_matrix.M[1][2],exp);
  fprintf(output," |                                      |\n");
  fprintf(output," |  %10.3f  %10.3f  %10.3f  |\n",
		  my_matrix.M[2][0],my_matrix.M[2][1],my_matrix.M[2][2]);
  fprintf(output," |                                      |\n");
  fprintf(output," +--                                  --+\n");
};

/* ----------------------------------------------------------------- */
void output_S_SB_matrices(FILE *output,int paginate)
{
  int i;

  for (i=1;i<=total_layers;i++)
  {
    if (paginate) clrscr();
    fprintf(output,"\n");
    fprintf(output,"Layer %2d S (special orthotropic) matrix (0 deg) :\n",i);
    print_matrix(output,S[i]," ");
    fprintf(output,"         _\n");
    fprintf(output,"Layer %2d S (general orthotropic) matrix (%6.1f deg) :\n",i,layer_angle[i]);
    print_matrix(output,SB[i]," ");
    if (paginate) Get_ENTER();
  };
}

/* ----------------------------------------------------------------- */
void output_Q_QB_matrices(FILE *output,int paginate)
{
  int i;

  for (i=1;i<=total_layers;i++)
  {
    if (paginate) clrscr();
    fprintf(output,"\n");
    fprintf(output,"Layer %2d Q (special orthotropic) matrix (0 deg) :\n",i);
    print_matrix(output,Q[i]," ");
    fprintf(output,"         _\n");
    fprintf(output,"Layer %2d Q (general orthotropic) matrix (%6.1f deg) :\n",i,layer_angle[i]);
    print_matrix(output,QB[i]," ");
    if (paginate) Get_ENTER();
  };
}

/* ----------------------------------------------------------------- */
void compute_KT()
{
  KT = 1.0 +
      sqrt ( fabs(
	(2.0 / A.M[1][1] ) *
	( sqrt(A.M[0][0]*A.M[1][1]) -
	  ( ( A.M[0][1]+A.M[0][0]*A.M[1][1] - sqr(A.M[0][1])) / (2.0 * A.M[2][2]) )
	)
	   ));
};

/* ----------------------------------------------------------------- */
void output_laminate_properties(FILE *output,int paginate)
{
  int i;

  /* output laminate properties */
  if (paginate) clrscr();
  fprintf(output,"Laminate properties  : %s\n",laminate_def);
  fprintf(output,"----------------------\n");
  fprintf(output,"number of layers     = %10d\n",total_layers);
  fprintf(output,"total thickness      = %10.3f\n",total_thickness);
  fprintf(output,"position of midplane = %10.3f\n",midplane);
  fprintf(output,"KT (stress con fact) = %10.3f\n",KT);
  if (paginate) Get_ENTER();

  /* output material properties for each layer */
  for (i=1;i<=total_layers;i++)
  {
    if (paginate) clrscr();
    fprintf(output,"Lamina properties for layer %2d :\n",i);
    fprintf(output,"--------------------------------\n");
    fprintf(output," E11       = %10.3f\n",E11[i]);
    fprintf(output," E22       = %10.3f\n",E22[i]);
    fprintf(output," NU12      = %10.3f\n",NU12[i]);
    fprintf(output," NU21      = %10.3f\n",NU21[i]);
    fprintf(output," G12       = %10.3f\n",G12[i]);
    fprintf(output," angle     = %10.3f (deg)\n",layer_angle[i]);
    fprintf(output," thickness = %10.3f\n",thickness[i]);
    /*
    fprintf(output," position of top     = %10.3f\n",top[i]);
    fprintf(output," position of bottom  = %10.3f\n",bottom[i]);
    */
    fprintf(output," midplane to top     = %10.3f\n",midplane_to_top[i]);
    fprintf(output," midplane to bottom  = %10.3f\n\n",midplane_to_bottom[i]);
    if (paginate) Get_ENTER();
  };
}

/* ----------------------------------------------------------------- */
void output_ABD_matrices(FILE *output,int paginate)
{
  if (paginate) clrscr();
  fprintf(output,"A matrix for laminate %s :\n",laminate_def);
  print_matrix(output,A," x 10^6");
  if (paginate) Get_ENTER();

  if (paginate) clrscr();
  fprintf(output,"B matrix for laminate %s :\n",laminate_def);
  print_matrix(output,B," x 10^3");
  if (paginate) Get_ENTER();

  if (paginate) clrscr();
  fprintf(output,"D matrix for laminate %s :\n",laminate_def);
  print_matrix(output,D," ");
  if (paginate) Get_ENTER();
}

/* ----------------------------------------------------------------- */
int output_menu()
{
  int choice = 0;
  char choice_str[10];

  while ( (choice<1) || (choice>11) )
  {
    clrscr();
    printf("\n ------ Output Matrices ------\n\n");
    printf(" 1) Output (to disk file) lamina/laminate properties.\n");
    printf(" 2) Output (to disk file) S and SB matrices.\n");
    printf(" 3) Output (to disk file) Q and QB matrices.\n");
    printf(" 4) Output (to disk file) ABD matrices.\n");
    printf(" 5) Output (to disk file) all data.\n");
    printf("\n");
    printf(" 6) Output (to screen) lamina/laminate properties.\n");
    printf(" 7) Output (to screen) S and SB matrices.\n");
    printf(" 8) Output (to screen) Q and QB matrices.\n");
    printf(" 9) Output (to screen) ABD matrices.\n");
    printf("10) Output (to screen) all data.\n");
    printf("\n");
    printf("11) Return to main menu.\n\n");
    printf("\n");
    printf(" Choice (1-11) :");
    scanf("%s",choice_str);
    choice=atoi(choice_str);
    printf("\n");
    getc(stdin);
  };

  return choice;
}

/* ----------------------------------------------------------------- */
void Get_Filename(char *filename)
{
  char fname[255];

  printf("\nEnter filename for output :");
  scanf("%s",fname);
  printf("\n");
  sprintf(filename,"%s",fname);
};


/* ----------------------------------------------------------------- */
void Output_Data()
{
  FILE *output;
  int  selection=0;
  char filename[255];

  compute_S_matrices();
  compute_SB_matrices();
  compute_Q_matrices();
  compute_QB_matrices();
  compute_Hj_values();
  create_laminate_def();
  compute_ABD_matrices();
  compute_KT();

  while (selection != 11)
  {
    selection=output_menu();
    switch (selection)
    {
      case 1: Get_Filename(filename);
	      output=fopen(filename,"w");
	      output_laminate_properties(output,FALSE);
	      fclose(output);
	      break;

      case 6: output=stdout;
	      output_laminate_properties(output,TRUE);
	      break;

      case 2: Get_Filename(filename);
	      output=fopen(filename,"w");
	      output_S_SB_matrices(output,FALSE);
	      fclose(output);
	      break;

      case 7: output=stdout;
	      output_S_SB_matrices(output,TRUE);
	      break;

      case 3: Get_Filename(filename);
	      output=fopen(filename,"w");
	      output_Q_QB_matrices(output,FALSE);
	      fclose(output);
	      break;

      case 8: output=stdout;
	      output_Q_QB_matrices(output,TRUE);
	      break;

      case 4: Get_Filename(filename);
	      output=fopen(filename,"w");
	      output_ABD_matrices(output,FALSE);
	      fclose(output);
	      break;

      case 9: output=stdout;
	      output_ABD_matrices(output,TRUE);
	      break;

      case 5: Get_Filename(filename);
	      output=fopen(filename,"w");
	      output_laminate_properties(output,FALSE);
	      output_S_SB_matrices(output,FALSE);
	      output_Q_QB_matrices(output,FALSE);
	      output_ABD_matrices(output,FALSE);
	      fclose(output);
	      break;

     case 10: output=stdout;
	      output_laminate_properties(output,TRUE);
	      output_S_SB_matrices(output,TRUE);
	      output_Q_QB_matrices(output,TRUE);
	      output_ABD_matrices(output,TRUE);
	      break;

     default: break;
    };
  };
}


/* ----------------------------------------------------------------- */
int main_menu()
{
  int choice = 0,
      max = maxlayers-1;

  char choice_str[10];

  while ( (choice<1) || (choice>7) )
  {
    clrscr();
    printf("\n ------ Composite Computer ------\n\n");
    printf(" 1) Enter total number of layers ( current=%d | max=%d ).\n",total_layers,max);
    printf(" 2) Enter/display lamina properties.\n");
    printf(" 3) Enter stress/strains on lamina.\n");
    printf(" 4) Save laminate to disk file.\n");
    printf(" 5) Read laminate from disk file.\n");
    printf(" 6) Output laminate properties and matrices to screen or file.\n");
    printf("\n");
    printf(" 7) Exit.\n\n");
    printf(" Choice (1-7) :");
    scanf("%s",choice_str);
    choice=atoi(choice_str);
    printf("\n");
    getc(stdin);
  };

  return choice;
}

/* ----------------------------------------------------------------- */
int lamina_menu()
{
  int choice = 0;
  char choice_str[10];

  while ( (choice<1) || (choice>11) )
  {
    clrscr();
    printf("\n ------- Material/Lamina Input ------\n\n");
    printf("  1) Change current layer (%d).\n",current_layer);
    printf("  2) Copy properties from another layer.\n");
    printf("  3) Enter layer (%d) properties.\n",current_layer);
    printf("  4) Enter layer (%d) E11       = %10.3f\n",current_layer,E11[current_layer]);
    printf("  5) Enter layer (%d) E22       = %10.3f\n",current_layer,E22[current_layer]);
    printf("  6) Enter layer (%d) NU12      = %10.3f\n",current_layer,NU12[current_layer]);
    printf("  7) Enter layer (%d) NU21      = %10.3f\n",current_layer,NU21[current_layer]);
    printf("  8) Enter layer (%d) G12       = %10.3f\n",current_layer,G12[current_layer]);
    printf("  9) Enter layer (%d) angle     = %10.3f (deg)\n",current_layer,layer_angle[current_layer]);
    printf(" 10) Enter layer (%d) thickness = %10.3f\n",current_layer,thickness[current_layer]);
    printf("\n");
    printf(" 11) Return to main menu.\n\n");
    printf(" Choice (1-11) :");
    scanf("%s",choice_str);
    choice=atoi(choice_str);
    printf("\n");
    getc(stdin);
  };

  return choice;
}

/* ----------------------------------------------------------------- */
void Get_E11(int layer)
{
  double value;
  char value_str[10];

  printf("\nLayer %d : Enter new E11  (old E11  = %10.3f):",layer,E11[layer]);
  scanf("%s",value_str);
  printf("\n");
  value=atof(value_str);
  E11[layer]=value;
};

/* ----------------------------------------------------------------- */
void Get_E22(int layer)
{
  double value;
  char value_str[10];

  printf("\nLayer %d : Enter new E22  (old E22  = %10.3f):",layer,E22[layer]);
  scanf("%s",value_str);
  printf("\n");
  value=atof(value_str);
  E22[layer]=value;
};

 /* ----------------------------------------------------------------- */
void Get_NU12(int layer)
{
  double value;
  char value_str[10];

  printf("\nLayer %d : Enter new NU12 (old NU12 = %10.3f):",layer,NU12[layer]);
  scanf("%s",value_str);
  printf("\n");
  value=atof(value_str);
  NU12[layer]=value;
};

/* ----------------------------------------------------------------- */
void Get_NU21(int layer)
{
  double value;
  char value_str[10];

  printf("\nLayer %d : Enter new NU21 (old NU21 = %10.3f):",layer,NU21[layer]);
  scanf("%s",value_str);
  printf("\n");
  value=atof(value_str);
  NU21[layer]=value;
};

/* ----------------------------------------------------------------- */
void Get_G12(int layer)
{
  double value;
  char value_str[10];

  printf("\nLayer %d : Enter new G12  (old G12  = %10.3f):",layer,G12[layer]);
  scanf("%s",value_str);
  printf("\n");
  value=atof(value_str);
  G12[layer]=value;
};

/* ----------------------------------------------------------------- */
void Get_Angle(int layer)
{
  double value;
  char value_str[10];

  printf("\nLayer %d : Enter new angle (old angle = %10.3f deg):",layer,layer_angle[layer]);
  scanf("%s",value_str);
  printf("\n");
  value=atof(value_str);
  layer_angle[layer]=value;
};

/* ----------------------------------------------------------------- */
void Get_Thickness(int layer)
{
  double value;
  char value_str[10];

  printf("\nLayer %d : Enter new thickness (old thickness = %10.3f):",layer,thickness[layer]);
  scanf("%s",value_str);
  printf("\n");
  value=atof(value_str);
  thickness[layer]=value;
};

/* ----------------------------------------------------------------- */
void Get_Total_Layers()
{
  int value;
  char value_str[10];

  printf("\nEnter new number of layers (old total = %d):",total_layers);
  scanf("%s",value_str);
  printf("\n");
  value=atoi(value_str);
  if ((value>0) && (value<maxlayers))
    total_layers=value;
};

/* ----------------------------------------------------------------- */
void Lamina_Input()
{
  int  selection=0,
       value,
       copy_layer;
  char value_str[10];

  while (selection!=11)
  {
    selection=lamina_menu();
    switch (selection)
    {
      case 1: printf("\nEnter new layer; legal range=(1-%d) :",total_layers);
	      scanf("%s",value_str);
	      printf("\n");
	      value=atoi(value_str);
	      if ( (value>0) && (value<=total_layers) )
		current_layer=value;
	      else
	      {
		printf("Illegal value for new layer (%s).\n",value_str);
		Get_ENTER(TRUE);
	      };
	      break;

      case 2: printf("\nEnter layer to copy values from; legal range=(1-%d) :",total_layers);
	      scanf("%s",value_str);
	      printf("\n");
	      value=atoi(value_str);
	      if ( (value>0) && (value<=total_layers) )
	      {
		copy_layer=value;
		printf("\nCopy all values from layer %d to layer %d (y/n):",copy_layer,current_layer);
		scanf("%s",value_str);
		if (tolower(value_str[0]) == 'y')
		{
		  E11[current_layer]         = E11[copy_layer];
		  E22[current_layer]         = E22[copy_layer];
		  NU12[current_layer]        = NU12[copy_layer];
		  NU21[current_layer]        = NU21[copy_layer];
		  G12[current_layer]         = G12[copy_layer];
		  layer_angle[current_layer] = layer_angle[copy_layer];
		  thickness[current_layer]   = thickness[copy_layer];
		};
	      }
	      else
	      {
		printf("\nIllegal value for copy layer (%s).\n",value_str);
		Get_ENTER(TRUE);
	      };
	      break;
      case 3: Get_E11(current_layer);
	      Get_E22(current_layer);
	      Get_NU12(current_layer);
	      Get_NU21(current_layer);
	      Get_G12(current_layer);
	      Get_Angle(current_layer);
	      Get_Thickness(current_layer);
	      break;
      case 4: Get_E11(current_layer);
	      break;
      case 5: Get_E22(current_layer);
	      break;
      case 6: Get_NU12(current_layer);
	      break;
      case 7: Get_NU21(current_layer);
	      break;
      case 8: Get_G12(current_layer);
	      break;
      case 9: Get_Angle(current_layer);
	      break;
      case 10: Get_Thickness(current_layer);
	       break;
      default: break;
    };
  };
}

/* ----------------------------------------------------------------- */
int load_menu()
{
  int  choice=0;
  char choice_str[10];

  while ( (choice<1) || (choice>4) )
  {
     clrscr();
     printf("\n --------- Stress/Strain Input ------------\n\n");
     printf(" 1) Change current layer (%d).\n",current_layer);
     printf(" 2) Enter stresses on layer %d -> Output strains from known stresses.\n",current_layer);
     printf(" 3) Enter strains on layer %d -> Output stresses from known strains.\n",current_layer);
     printf("\n");
     printf(" 4) Return to main menu.\n\n");
     printf(" Choice (1-4) :");
     scanf("%s",choice_str);
     choice=atoi(choice_str);
     printf("\n");
     getc(stdin);
  };

  return choice;
}

/* ----------------------------------------------------------------- */
void output_strains(int cl,
		    double stressXX,
		    double stressYY,
		    double stressXY,
		    double strainXX,
		    double strainYY,
		    double strainXY)
{
  clrscr();
  printf("                            _\n");
  printf("     STRESSES               S (COMPLIANCE MATRIX)               STRAINS\n");
  printf("+--           --+   +--                              --+   +--           --+\n");
  printf("|               |   |                                  |   |               |\n");
  printf("|  x=%10.3f |   | %10.3f %10.3f %10.3f |   |  x=%10.3f |\n",
    stressXX,SB[cl].M[0][0],SB[cl].M[0][1],SB[cl].M[0][2],strainXX);
  printf("|               |   |                                  |   |               |\n");
  printf("|  y=%10.3f | x | %10.3f %10.3f %10.3f | = |  y=%10.3f |\n",
    stressYY,SB[cl].M[1][0],SB[cl].M[1][1],SB[cl].M[1][2],strainYY);
  printf("|               |   |                                  |   |               |\n");
  printf("| xy=%10.3f |   | %10.3f %10.3f %10.3f |   | xy=%10.3f |\n",
    stressXY,SB[cl].M[2][0],SB[cl].M[2][1],SB[cl].M[2][2],strainXY);
  printf("|               |   |                                  |   |               |\n");
  printf("+--           --+   +--                              --+   +--           --+\n");
  Get_ENTER();
};

/* ----------------------------------------------------------------- */
void output_stresses(int cl,
		     double strainXX,
		     double strainYY,
		     double strainXY,
		     double stressXX,
		     double stressYY,
		     double stressXY)
{
  clrscr();
  printf("                            _\n");
  printf("     STRAINS                Q (STIFFNESS MATRIX)                STRESSES\n");
  printf("+--           --+   +--                              --+   +--           --+\n");
  printf("|               |   |                                  |   |               |\n");
  printf("|  x=%10.3f |   | %10.3f %10.3f %10.3f |   |  x=%10.3f |\n",
    strainXX,QB[cl].M[0][0],QB[cl].M[0][1],QB[cl].M[0][2],stressXX);
  printf("|               |   |                                  |   |               |\n");
  printf("|  y=%10.3f | x | %10.3f %10.3f %10.3f | = |  y=%10.3f |\n",
    strainYY,QB[cl].M[1][0],QB[cl].M[1][1],QB[cl].M[1][2],stressYY);
  printf("|               |   |                                  |   |               |\n");
  printf("| xy=%10.3f |   | %10.3f %10.3f %10.3f |   | xy=%10.3f |\n",
    strainXY,QB[cl].M[2][0],QB[cl].M[2][1],QB[cl].M[2][2],stressXY);
  printf("|               |   |                                  |   |               |\n");
  printf("+--           --+   +--                              --+   +--           --+\n");
  Get_ENTER();
};


/* ----------------------------------------------------------------- */
void Load_Input()
{
  int    selection=0,
	 value,
	 copy_layer,
	 cl;
  char   value_str[15];
  double stressXX,stressYY,stressXY,
	 strainXX,strainYY,strainXY;

  while (selection!=4)
  {
    selection=load_menu();
    switch (selection)
    {
      case 1: printf("\nEnter new layer; legal range=(1-%d) :",total_layers);
	      scanf("%s",value_str);
	      printf("\n");
	      value=atoi(value_str);
	      if ( (value>0) && (value<=total_layers) )
		current_layer=value;
	      else
	      {
		printf("Illegal value for new layer (%s).\n",value_str);
		Get_ENTER();
	      };
	      break;

      case 2: printf("Enter stress XX :");
	      scanf("%s",value_str);
	      printf("\n");
	      stressXX=atof(value_str);

	      printf("Enter stress YY :");
	      scanf("%s",value_str);
	      printf("\n");
	      stressYY=atof(value_str);

	      printf("Enter stress XY :");
	      scanf("%s",value_str);
	      printf("\n");
	      stressXY=atof(value_str);

	      getc(stdin);
	      compute_S_matrices();
	      compute_SB_matrices();
	      cl=current_layer;
	      strainXX=SB[cl].M[0][0]*stressXX+
		       SB[cl].M[0][1]*stressYY+
		       SB[cl].M[0][2]*stressXY;

	      strainYY=SB[cl].M[1][0]*stressXX+
		       SB[cl].M[1][1]*stressYY+
		       SB[cl].M[1][2]*stressXY;

	      strainXY=SB[cl].M[2][0]*stressXX+
		       SB[cl].M[2][1]*stressYY+
		       SB[cl].M[2][2]*stressXY;

	      output_strains(cl,stressXX,stressYY,stressXY,
				strainXX,strainYY,strainXY);

	      break;

      case 3: printf("Enter strain XX :");
	      scanf("%s",value_str);
	      printf("\n");
	      strainXX=atof(value_str);

	      printf("Enter strain YY :");
	      scanf("%s",value_str);
	      printf("\n");
	      strainYY=atof(value_str);

	      printf("Enter strain XY :");
	      scanf("%s",value_str);
	      printf("\n");
	      strainXY=atof(value_str);

	      getc(stdin);
	      compute_Q_matrices();
	      compute_QB_matrices();
	      cl=current_layer;
	      stressXX=QB[cl].M[0][0]*strainXX+
		       QB[cl].M[0][1]*strainYY+
		       QB[cl].M[0][2]*strainXY;

	      stressYY=QB[cl].M[1][0]*strainXX+
		       QB[cl].M[1][1]*strainYY+
		       QB[cl].M[1][2]*strainXY;

	      stressXY=QB[cl].M[2][0]*strainXX+
		       QB[cl].M[2][1]*strainYY+
		       QB[cl].M[2][2]*strainXY;

	      output_stresses(cl,strainXX,strainYY,strainXY,
				 stressXX,stressYY,stressXY);

	      break;
     default: break;
    };
  }
};

/* ----------------------------------------------------------------- */
void Save_Laminate_Data()
{
  FILE *lam_file;
  char filename[255];
  int  i;

   printf("\nEnter filename to save current defined laminate into :\n");
   scanf("%s",filename);
   printf("\n");

   lam_file=fopen(filename,"w");
   if ( ! lam_file )
   {
     printf("\nCould not open (%s) to write.\n",filename);
     Get_ENTER(TRUE);
     return;
   }

   fprintf(lam_file,"total_layers=%2d\n",total_layers);
   for (i=1;i<=total_layers;i++)
   {
     fprintf(lam_file,"layer=%2d :   E11=%10.3f\n",i,E11[i]);
     fprintf(lam_file,"layer=%2d :   E22=%10.3f\n",i,E22[i]);
     fprintf(lam_file,"layer=%2d :  NU12=%10.3f\n",i,NU12[i]);
     fprintf(lam_file,"layer=%2d :  NU21=%10.3f\n",i,NU21[i]);
     fprintf(lam_file,"layer=%2d :   G12=%10.3f\n",i,G12[i]);
     fprintf(lam_file,"layer=%2d : angle=%10.3f\n",i,layer_angle[i]);
     fprintf(lam_file,"layer=%2d : thick=%10.3f\n",i,thickness[i]);
   };
   fprintf(lam_file,"END OF FILE.\n\n");
   fclose(lam_file);
}


/* ----------------------------------------------------------------- */
void Retrieve_Laminate_Data()
{
  FILE *lam_file;
  char c,filename[255],
       line[100],tag[6],
       value_str[11],
       layer_str[3],
       eofline[15];

  int  done,i,j,this_layer;

  printf("\nEnter filename to read defined laminate from :\n");
  scanf("%s",filename);
  printf("\n");
  getc(stdin);

  sprintf(eofline,"END OF FILE.");
  lam_file=fopen(filename,"r");
  if (! lam_file)
  {
    printf("\nCould not open (%s) to read.\n",filename);
    Get_ENTER(TRUE);
    return;
  }

  done=FALSE;
  while (! done)
  {
    j=0;
    c=' ';
    while ((c!='\n') && (c!='\0'))
    {
      c=fgetc(lam_file);
      line[j]=c;
      j++;
    };
    line[j]='\0';

    if (strcmp(line,"total_layers=") > 0)
    {
      value_str[0]=line[13];
      value_str[1]=line[14];
      value_str[2]='\0';
      total_layers=atoi(value_str);
    }
    else if (strcmp(line,"layer=") > 0)
    {
      layer_str[0]=line[6];
      layer_str[1]=line[7];
      layer_str[2]='\0';
      this_layer=atoi(layer_str);

      for (i=0;i<5;i++)  tag[i]=line[i+11]; tag[5]='\0';
      for (i=0;i<10;i++) value_str[i]=line[i+17]; value_str[10]='\0';

      if (strcmp(tag,"  E11")==0) E11[this_layer]=atof(value_str);
      if (strcmp(tag,"  E22")==0) E22[this_layer]=atof(value_str);
      if (strcmp(tag," NU12")==0) NU12[this_layer]=atof(value_str);
      if (strcmp(tag," NU21")==0) NU21[this_layer]=atof(value_str);
      if (strcmp(tag,"  G12")==0) G12[this_layer]=atof(value_str);
      if (strcmp(tag,"thick")==0) thickness[this_layer]=atof(value_str);
      if (strcmp(tag,"angle")==0) layer_angle[this_layer]=atof(value_str);
    };

    done=TRUE; for (j=0;j<11;j++) done=(done && (line[i]==eofline[i]));
    if (feof(lam_file)) done=TRUE;
  };
  fclose(lam_file);
}

/* ----------------------------------------------------------------- */
void main(argc,argv)
  int argc;
  char *argv[];
{
  FILE *output;
  int i,j,k,
      main_sel=0;

  /* initialize all values/matrices to 0.0 */
  for (k=0;k<maxlayers;k++)
  {
    for (i=0;i<3;i++)
    for (j=0;j<3;j++)
    {
      A.M[i][j]           = 0.0;
      B.M[i][j]           = 0.0;
      D.M[i][j]           = 0.0;
      Q[k].M[i][j]        = 0.0;
      QB[k].M[i][j]       = 0.0;
      S[k].M[i][j]        = 0.0;
      SB[k].M[i][j]       = 0.0;
    }
    E11[k]                = 0.0;
    E22[k]                = 0.0;
    NU12[k]               = 0.0;
    NU21[k]               = 0.0;
    G12[k]                = 0.0;
    layer_angle[k]        = 0.0;
    thickness[k]          = 0.0;
    top[k]                = 0.0;
    bottom[k]             = 0.0;
    midplane_to_bottom[k] = 0.0;
    midplane_to_top[k]    = 0.0;
  }

  while (main_sel != 7)
  {
    main_sel=main_menu();
    switch (main_sel)
    {
      case 1: Get_Total_Layers();
	      break;

      case 2: if (total_layers) Lamina_Input();
	      else
	      {
		printf("\nTotal layers = 0 : Define number of layers.\n");
		Get_ENTER();
	      };
	      break;

      case 3: if (total_layers) Load_Input();
	      else
	      {
		printf("\nTotal layers = 0 : Define number of layers.\n");
		Get_ENTER();
	      };
	      break;

      case 4: if (total_layers) Save_Laminate_Data();
	      else
	      {
		printf("\nTotal layers = 0 : Define number of layers.\n");
		Get_ENTER();
	      };
	      break;

      case 5: Retrieve_Laminate_Data();
	      break;

      case 6: if (total_layers) Output_Data();
	      else
	      {
		printf("\nTotal layers = 0 : Define number of layers.\n");
		Get_ENTER();
	      };
	      break;

     default: break;
    };
  };

}
